from typing import Optional

from sqlalchemy import Column, String
from sqlalchemy.orm import Session

# from edulib.models import BaseModelOrm
# from auth.utils import get_password_hash

from edulib.models.base import BaseModelOrm


class UserOrm(BaseModelOrm):
    __tablename__ = "user"

    username = Column(String, nullable=False, unique=True)
    hashed_password = Column(String, nullable=False, unique=True)
    email = Column(String, nullable=False, unique=True)

    def __repr__(self) -> str:
        return (
            f"UserOrm(id={self.id!r}, username={self.username!r}, "
            f"hashed_password={self.hashed_password!r}, email={self.email!r})"
        )

    @classmethod
    def get_user_by_username(
        cls, db: Session, username: str
    ) -> Optional["UserOrm"]:
        return db.query(cls).filter(cls.username == username).first()

    @classmethod
    def create(cls, db: Session, kwargs: dict) -> "UserOrm":
        password = kwargs.pop("password")
        kwargs["hashed_password"] = get_password_hash(password)

        return super().create(db, kwargs)

    @classmethod
    def update(cls, db: Session, id_: int, kwargs: dict) -> "UserOrm":
        password = kwargs.pop("password")
        kwargs["hashed_password"] = get_password_hash(password)

        return super().update(db, id_, kwargs)
